searchNodes=[{"ref":"I18nHelpers.Ecto.TranslatableSchema.html","title":"I18nHelpers.Ecto.TranslatableSchema","type":"behaviour","doc":"Provides macros for defining translatable fields and associations. This module&#39;s purpose is to provide the I18nHelpers.Ecto.Translator module with a way to access the list of fields and associations from the Ecto Schema that needs to be translated, and with virtual fields allowing to store the translations for the current locale. __using__\\1 this module provides the caller module with two functions: get_translatable_fields\\0 and get_translatable_assocs\\0 listing all the translatable fields and the translatable associations respectively. Fields that are to be translated are expected to hold maps field :title, :map where each key represents a locale and each value contains the text for that locale. Below is an example of such map: %{ &quot;en&quot; =&gt; &quot;My Favorite Books&quot;, &quot;fr&quot; =&gt; &quot;Mes Livres Préférés&quot;, &quot;nl&quot; =&gt; &quot;Mijn Lievelingsboeken&quot;, &quot;en-GB&quot; =&gt; &quot;My Favourite Books&quot; } Each of those fields must come with a virtual field which is used to hold the translation for the current locale. field :title, :map field :translated_title, :string, virtual: true Such a translatable field must be included in the translatable fields list: def get_translatable_fields, do: [:title] This module provides the macro translatable_field\\1 which allows to execute those three steps above (add the field as :map, add the virtual field and add the field to the translatable fields list) in one line: translatable_field :title Macros marking associations as translatable are also provided: translatable_belongs_to\\2 translatable_has_many\\2 translatable_has_one\\2 translatable_many_to_many\\3 The macros above add the given association field name to the translatable associations list, which is accessible with get_translatable_assocs\\0."},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#c:get_translatable_assocs/0","title":"I18nHelpers.Ecto.TranslatableSchema.get_translatable_assocs/0","type":"callback","doc":""},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#c:get_translatable_fields/0","title":"I18nHelpers.Ecto.TranslatableSchema.get_translatable_fields/0","type":"callback","doc":""},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#translatable_belongs_to/2","title":"I18nHelpers.Ecto.TranslatableSchema.translatable_belongs_to/2","type":"macro","doc":"Defines a translatable belongs_to association. The macro will add the given field name into the translatable associations list."},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#translatable_field/1","title":"I18nHelpers.Ecto.TranslatableSchema.translatable_field/1","type":"macro","doc":"Defines a translatable field on the schema. This macro will generate two fields: a field with the given name and type :map and a virtual field with the given name prepended by &quot;translated_&quot; and type :string. For example translatable_field :title will generate field :title, :map field :translated_title, :string, virtual: true The macro will add the given field name into the translatable fields list."},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#translatable_has_many/2","title":"I18nHelpers.Ecto.TranslatableSchema.translatable_has_many/2","type":"macro","doc":"Defines a translatable has_many association. The macro will add the given field name into the translatable associations list."},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#translatable_has_one/2","title":"I18nHelpers.Ecto.TranslatableSchema.translatable_has_one/2","type":"macro","doc":"Defines a translatable has_one association. The macro will add the given field name into the translatable associations list."},{"ref":"I18nHelpers.Ecto.TranslatableSchema.html#translatable_many_to_many/3","title":"I18nHelpers.Ecto.TranslatableSchema.translatable_many_to_many/3","type":"macro","doc":"Defines a translatable many_to_many association. The macro will add the given field name into the translatable associations list."},{"ref":"I18nHelpers.Ecto.Translator.html","title":"I18nHelpers.Ecto.Translator","type":"module","doc":""},{"ref":"I18nHelpers.Ecto.Translator.html#set_opts/1","title":"I18nHelpers.Ecto.Translator.set_opts/1","type":"function","doc":""},{"ref":"I18nHelpers.Ecto.Translator.html#translate/3","title":"I18nHelpers.Ecto.Translator.translate/3","type":"function","doc":"Translates an Ecto struct, a list of Ecto structs or a map containing translations. Translating an Ecto struct for a given locale consists of the following steps: Get the list of the fields that need to be translated from the Schema. The Schema must contain a get_translatable_fields\\0 function returning a list of those fields. Get the text for the given locale and store it into a virtual field. The Schema must provide, for each translatable field, a corresponding virtual field in order to store the translation. Get the list of the associations that also need to be translated from the Schema. The Schema must contain a get_translatable_assocs\\0 function returning a list of those associations. Repeat step 1. for each associated Ecto struct."},{"ref":"I18nHelpers.Form.InputHelpers.html","title":"I18nHelpers.Form.InputHelpers","type":"module","doc":"Provides view helpers to render HTML input fields for text that must be provided in multiple languages. The multilingual texts passed to the form (usually in a changeset) are expected to be maps where each key represents a locale and each value contains the text for that locale. For example: %{ &quot;en&quot; =&gt; &quot;hello world&quot;, &quot;fr&quot; =&gt; &quot;bonjour monde&quot;, &quot;nl&quot; =&gt; &quot;hallo wereld&quot; }"},{"ref":"I18nHelpers.Form.InputHelpers.html#translated_text_input/4","title":"I18nHelpers.Form.InputHelpers.translated_text_input/4","type":"function","doc":"Renders a text input HTML element for the given locale. Additional HTML attributes can be provided through opts argument."},{"ref":"I18nHelpers.Form.InputHelpers.html#translated_text_inputs/4","title":"I18nHelpers.Form.InputHelpers.translated_text_inputs/4","type":"function","doc":"Renders multiple text input HTML elements for the given locales (one for each locale). Options The options allow providing additional HTML attributes, as well as: :labels - an anonymous function returning the label for each generated input; the locale is given as argument :wrappers - an anonymous function returning a custom wrapper for each generated input; the locale is given as argument Example translated_text_inputs(f, :title, [:en, :fr], labels: fn locale -&gt; content_tag(:i, locale) end, wrappers: fn _locale -&gt; {:div, class: &quot;translated-input-wrapper&quot;} end )"},{"ref":"I18nHelpers.Form.InputHelpers.html#translated_textarea/4","title":"I18nHelpers.Form.InputHelpers.translated_textarea/4","type":"function","doc":"Renders a textarea HTML element for the given locale. Additional HTML attributes can be provided through opts argument."},{"ref":"I18nHelpers.Form.InputHelpers.html#translated_textareas/4","title":"I18nHelpers.Form.InputHelpers.translated_textareas/4","type":"function","doc":"Renders multiple textarea HTML elements for the given locales (one for each locale). For options, see translated_text_inputs/4"},{"ref":"I18nHelpers.Plugs.PutLocaleFromDomain.html","title":"I18nHelpers.Plugs.PutLocaleFromDomain","type":"module","doc":"Plug to fetch the locale from the URL&#39;s subdomain; assigns the locale to the Connection and sets the Gettext locale. This plug is useful if you have URLs similar to: https://mon-super-site.example/bonjour https://mijn-geweldige-website.example/hallo https://mi-gran-sitio.example/hola https://my-awesome-website.example/hello Options :domains_locales_map - map where each key represents a domain and each value contains the locale to be used for that domain :default_locale - locale to be used if no locale was found in the URL :allowed_locales - a list of allowed locales. If no locale was found, use the :default locale if specified, otherwise raise an error. :domains_locales_map is a mandatory option. Below is an example of a map it can hold: %{ &quot;mon-super-site.example&quot; =&gt; &quot;fr&quot;, &quot;mijn-geweldige-website.example&quot; =&gt; &quot;nl&quot;, &quot;mi-gran-sitio.example&quot; =&gt; &quot;es&quot;, &quot;my-awesome-website.example&quot; =&gt; &quot;en&quot; }"},{"ref":"I18nHelpers.Plugs.PutLocaleFromDomain.html#call/2","title":"I18nHelpers.Plugs.PutLocaleFromDomain.call/2","type":"function","doc":""},{"ref":"I18nHelpers.Plugs.PutLocaleFromDomain.html#init/1","title":"I18nHelpers.Plugs.PutLocaleFromDomain.init/1","type":"function","doc":""},{"ref":"I18nHelpers.Plugs.PutLocaleFromPath.html","title":"I18nHelpers.Plugs.PutLocaleFromPath","type":"module","doc":"Plug to fetch the locale from the first segment of the URL&#39;s request path; assigns the locale to the Connection and sets the Gettext locale. This plug is useful if you have URLs similar to: https://example.com/fr/bonjour https://example.com/nl/hallo https://example.com/es/hola https://example.com/hello (default locale &quot;en&quot;) Options :default_locale - locale to be used if no locale was found in the URL :allowed_locales - a list of allowed locales. If no locale was found, use the :default locale if specified, otherwise raise an error."},{"ref":"I18nHelpers.Plugs.PutLocaleFromPath.html#call/2","title":"I18nHelpers.Plugs.PutLocaleFromPath.call/2","type":"function","doc":""},{"ref":"I18nHelpers.Plugs.PutLocaleFromPath.html#init/1","title":"I18nHelpers.Plugs.PutLocaleFromPath.init/1","type":"function","doc":""},{"ref":"I18nHelpers.Plugs.PutLocaleFromSubdomain.html","title":"I18nHelpers.Plugs.PutLocaleFromSubdomain","type":"module","doc":"Plug to fetch the locale from the URL&#39;s subdomain; assigns the locale to the Connection and sets the Gettext locale. This plug is useful if you have URLs similar to: https://fr.example.com/bonjour https://nl.example.com/hallo https://es.example.com/hola https://example.com/hello (default locale &quot;en&quot;) Options :default_locale - locale to be used if no locale was found in the URL :allowed_locales - a list of allowed locales. If no locale was found, use the :default locale if specified, otherwise raise an error."},{"ref":"I18nHelpers.Plugs.PutLocaleFromSubdomain.html#call/2","title":"I18nHelpers.Plugs.PutLocaleFromSubdomain.call/2","type":"function","doc":""},{"ref":"I18nHelpers.Plugs.PutLocaleFromSubdomain.html#init/1","title":"I18nHelpers.Plugs.PutLocaleFromSubdomain.init/1","type":"function","doc":""},{"ref":"readme.html","title":"I18n Helpers","type":"extras","doc":"I18n Helpers I18n Helpers are a set of tools to help you adding multilingual support to your Elixir application. 1. Ease the use of translations stored in database Translate your Ecto Schema structs (including all Schema associations, in one call) post = Repo.all(Post) |&gt; Repo.preload(:category) |&gt; Repo.preload(:comments) |&gt; Translator.translate(&quot;fr&quot;) assert post.translated_title == &quot;Le titre&quot; assert post.category.translated_name == &quot;La catégorie&quot; assert List.first(post.comments).translated_text == &quot;Un commentaire&quot; Provide a fallback locale Translator.translate(post, &quot;nl&quot;, fallback_locale: &quot;en&quot;) Handle missing translations (e.g. get notified) Translator.translate(post, &quot;en&quot;, handle_missing_translation: fn translations_map, locale -&gt; # add here your error handling stuff, # e.g. notify yourself that a translation is missing end) 2. Render multilingual field inputs in your Phoenix Form Render multilanguage inputs to work with Ecto Schema structs that need translations Render multilanguage inputs in one call with custom labels and wrappers to customize design 3. Fetch the locale from the URL Assign the locale to the connection and set the Gettext locale Fetch the locale from the request path e.g. example.com/en/hello, example.com/fr/bonjour, … Fetch the locale from the subdomain e.g. en.example.com/hello, fr.example.com/bonjour, … Fetch the locale from the domain name e.g. my-awesome-website.example/hello, mon-super-site.example/bonjour, … Implement a custom locale fetcher"},{"ref":"readme.html#translate-your-ecto-schema","title":"I18n Helpers - Translate your Ecto Schema","type":"extras","doc":"Translations must be stored in a JSON data type. *Note: if you prefer to store translations in separate database tables, then this library (at least the Ecto-related helpers) is not for you. Note however that, in my opinion, the pros having a JSON field compared to separate translation tables largely outweigh the cons; but I will not debate that here and let you Google that yourself to form your own opinion.* Each translatable field is stored in a map, where each key represents a locale and each value contains the text for that locale. Below is an example of such map: %{ &quot;en&quot; =&gt; &quot;My Favorite Books&quot;, &quot;fr&quot; =&gt; &quot;Mes Livres Préférés&quot;, &quot;nl&quot; =&gt; &quot;Mijn Lievelingsboeken&quot;, &quot;en-GB&quot; =&gt; &quot;My Favourite Books&quot; } What this helper does not Let&#39;s first clarify something important in order to understand what this library actually helps with and what it does not. Your translatable text field is essentially a map. In your schema, this translates to: field :title, :map Inserting/updating/deleting translations is not handled by this library, as nothing specific has to be done to perform those with Ecto.Repo on a :map field. What this library helps with, is extracting the translations from an Ecto struct and associated structs into virtual fields based on a given locale, fallback to a given fallback locale, and handling missing translations. See examples below. Setup your schema without macro defmodule MyApp.Post do @behaviour I18nHelpers.Ecto.TranslatableSchema use Ecto.Schema schema &quot;posts&quot; do field :title, :map field :translated_title, :string, virtual: true field :body, :map field :translated_body, :string, virtual: true has_many :comments, MyApp.Comment belongs_to :category, MyApp.Category end def get_translatable_fields, do: [:title, :body] def get_translatable_assocs, do: [:comments, :category] end with macro defmodule MyApp.Post do use Ecto.Schema use I18nHelpers.Ecto.TranslatableSchema schema &quot;posts&quot; do translatable_field :title translatable_field :body translatable_has_many :comments, MyApp.Comment translatable_belongs_to :category, MyApp.Category end end You may import :i18n_helpers&#39;s formatter configuration by importing i18n_helpers into your .formatter.exs file (this allows for example to keep translatable_field :title without parentheses when running mix format). [ import_deps: [:ecto, :phoenix, :i18n_helpers], #... ] The translatable fields in your migration file should also be of :map type: add :title, :map, null: false add :body, :map, null: false Translate your Ecto struct You will typically translate Schema structs after retrieving them from the database: alias I18nHelpers.Ecto.Translator alias MyApp.{Post, Repo} post = Repo.all(Post) |&gt; Translator.translate(&quot;fr&quot;) assert translated_post.translated_title == &quot;Le titre&quot; assert translated_post.translated_body == &quot;Le contenu&quot; assert translated_post.category.translated_name == &quot;La catégorie&quot; Note above that all the associated Schema structs have been translated as well. I prefer to perform translations in the Phoenix controller: Blog.get_post!(post_id) # suppose Blog is the context managing posts, comments, etc. |&gt; Blog.with_comments_assocs() |&gt; Blog.with_category_assoc() |&gt; Translator.translate(&quot;fr&quot;) Below is an example that shows more clearly the content of the structs and their translations: alias I18nHelpers.Ecto.Translator alias MyApp.{Category, Comment, Post} comments = [ %Comment{text: %{&quot;en&quot; =&gt; &quot;A comment&quot;, &quot;fr&quot; =&gt; &quot;Un commentaire&quot;}}, %Comment{text: %{&quot;en&quot; =&gt; &quot;Another comment&quot;, &quot;fr&quot; =&gt; &quot;Un autre commentaire&quot;}} ] category = %Category{name: %{&quot;en&quot; =&gt; &quot;The category&quot;, &quot;fr&quot; =&gt; &quot;La catégorie&quot;}} post = %Post{ title: %{&quot;en&quot; =&gt; &quot;The title&quot;, &quot;fr&quot; =&gt; &quot;Le titre&quot;}, body: %{&quot;en&quot; =&gt; &quot;The content&quot;, &quot;fr&quot; =&gt; &quot;Le contenu&quot;} } |&gt; Map.put(:comments, comments) |&gt; Map.put(:category, category) translated_post = Translator.translate(post, &quot;fr&quot;) assert translated_post.translated_title == &quot;Le titre&quot; assert translated_post.translated_body == &quot;Le contenu&quot; assert hd(translated_post.comments).translated_text == &quot;Un commentaire&quot; assert translated_post.category.translated_name == &quot;La catégorie&quot; You can also translate a single field: title = Translator.translate(post.title, &quot;fr&quot;) # post.title == %{&quot;en&quot; =&gt; &quot;The title&quot;, &quot;fr&quot; =&gt; &quot;Le titre&quot;} assert title == &quot;Le titre&quot; Locale and fallback locale If you do not specify the locale to translate to, the library will use the global Gettext default locale: config :gettext, :default_locale, &quot;fr&quot; # in your `mix.exs` config file title = Translator.translate(post.title) assert title == &quot;Le titre&quot; The global locale can be set through a Plug based on the website&#39;s host or path (see included plugs below). A fallback locale can be given as an option. In the example below, we try to translate the title in Dutch, but no translation in Dutch has been provided. The translator will then use the given fallback locale: title = Translator.translate(post.title, &quot;nl&quot;, fallback_locale: &quot;en&quot;) assert title == &quot;The title&quot; The default fallback locale is the global Gettext default locale. Handling missing translations You may provide a callback to handle missing translations: Translator.translate(%{&quot;fr&quot; =&gt; &quot;bonjour&quot;}, &quot;en&quot;, handle_missing_translation: fn translations_map, locale -&gt; # add here your error handling stuff, # e.g. notify yourself that a translation is missing assert translations_map == %{&quot;fr&quot; =&gt; &quot;bonjour&quot;} assert locale == &quot;en&quot; end ) It can be quite tedious to pass your custom callback function to every translate/3 call; you can avoid this by wrapping translate/3 in your own function, where you setup the commonly used options. You can then import it for every controller through MyAppWeb.controller/0. Below is an example where we want to raise an error when a translation is not found: defmodule MyTranslator do alias I18nHelpers.Ecto.Translator def translate(data_structure, locale \\\\ Gettext.get_locale(), opts \\\\ []) do handle_missing_translation = Keyword.get(opts, :handle_missing_translation, &amp;handle_missing_translation/2) opts = Keyword.put(opts, :handle_missing_translation, handle_missing_translation) Translator.translate(data_structure, locale, opts) end def handle_missing_translation(translations_map, locale) do raise &quot;missing translation for locale `\#{locale}` in \#{inspect(translations_map)}&quot; end end"},{"ref":"readme.html#phoenix-form-helpers","title":"I18n Helpers - Phoenix Form helpers","type":"extras","doc":"In order to render a form input that will work on one translation entry of an embedded translations map, you must write something similar as the following: &lt;%= text_input f, :title_en, name: &quot;post[title][en]&quot;, value: Map.get(f.data.title, &quot;en&quot;, &quot;&quot;) %&gt; The library provide helpers allowing you to add such input in a more concise and clean way: &lt;%= translated_text_input f, :title, :en %&gt; &lt;%= translated_textarea f, :title, :en %&gt; You may also render all the inputs (for all languages) for a field in one line: translated_text_inputs(f, :title, [:en, :fr]) translated_text_inputs(f, :title, MyApp.Gettext) # will call Gettext.known_locales/1 on given Gettext backend translated_textareas(f, :title, [:en, :fr]) If you need custom labels and styling, you may pass options allowing you to add labels and wrap the generated inputs with custom HTML elements: translated_text_inputs(f, :title, [:en, :fr], labels: fn locale -&gt; content_tag(:i, locale) end, wrappers: fn _locale -&gt; {:div, class: &quot;translated-input-wrapper&quot;} end )"},{"ref":"readme.html#fetch-the-locale-from-the-url","title":"I18n Helpers - Fetch the locale from the URL","type":"extras","doc":"The library provides a set of plugs with different strategies to fetch the locale from the URL. The plug will assign the locale to the Connection and set the Gettext locale. You can retrieve the locale from the request path: plug I18nHelpers.Plugs.PutLocaleFromPath, allowed_locales: [&quot;en&quot;, &quot;fr&quot;], default_locale: &quot;en&quot; See tests below: alias I18nHelpers.Plugs.PutLocaleFromPath options = PutLocaleFromPath.init(allowed_locales: [&quot;fr&quot;, &quot;nl&quot;], default_locale: &quot;en&quot;) conn = conn(:get, &quot;https://example.com/fr/bonjour&quot;) conn = PutLocaleFromPath.call(conn, options) assert conn.assigns == %{locale: &quot;fr&quot;} assert Gettext.get_locale == &quot;fr&quot; conn = conn(:get, &quot;https://example.com/hello&quot;) # locale is not specified in path, use `default_locale` conn = PutLocaleFromPath.call(conn, options) assert conn.assigns == %{locale: &quot;en&quot;} assert Gettext.get_locale == &quot;en&quot; Or from the subdomain: plug I18nHelpers.Plugs.PutLocaleFromSubdomain, allowed_locales: [&quot;en&quot;, &quot;fr&quot;], default_locale: &quot;en&quot; Tests: alias I18nHelpers.Plugs.PutLocaleFromSubdomain options = PutLocaleFromSubdomain.init(allowed_locales: [&quot;en&quot;, &quot;fr&quot;], default_locale: &quot;en&quot;) conn = conn(:get, &quot;https://fr.example.com/bonjour&quot;) conn = PutLocaleFromSubdomain.call(conn, options) assert conn.assigns == %{locale: &quot;fr&quot;} assert Gettext.get_locale == &quot;fr&quot; conn = conn(:get, &quot;https://example.com/hello&quot;) # locale is not specified in subdomain, use `default_locale` conn = PutLocaleFromSubdomain.call(conn, options) assert conn.assigns == %{locale: &quot;en&quot;} assert Gettext.get_locale == &quot;en&quot; Or from the domain: plug I18nHelpers.Plugs.PutLocaleFromDomain, domains_locales_map: %{ &quot;my-awesome-website.example&quot; =&gt; &quot;en&quot;, &quot;mon-super-site.example&quot; =&gt; &quot;fr&quot; }, allowed_locales: [&quot;en&quot;, &quot;fr&quot;], default_locale: &quot;en&quot; Tests: alias I18nHelpers.Plugs.PutLocaleFromDomain options = PutLocaleFromDomain.init( domains_locales_map: %{ &quot;my-awesome-website.example&quot; =&gt; &quot;en&quot;, &quot;mon-super-site.example&quot; =&gt; &quot;fr&quot; }, allowed_locales: [&quot;en&quot;, &quot;fr&quot;], default_locale: &quot;en&quot; ) conn = conn(:get, &quot;https://my-awesome-website.example/hello&quot;) conn = PutLocaleFromDomain.call(conn, options) assert conn.assigns == %{locale: &quot;en&quot;} assert Gettext.get_locale == &quot;en&quot; conn = conn(:get, &quot;https://mon-super-site.example/bonjour&quot;) conn = PutLocaleFromDomain.call(conn, options) assert conn.assigns == %{locale: &quot;fr&quot;} assert Gettext.get_locale == &quot;fr&quot; conn = conn(:get, &quot;https://another-domain.example/hello&quot;) # domain not found in `domains_locales_map`, use `default_locale` conn = PutLocaleFromDomain.call(conn, options) assert conn.assigns == %{locale: &quot;en&quot;} assert Gettext.get_locale == &quot;en&quot; or from your custom function: alias I18nHelpers.Plugs.PutLocale defp find_locale(conn) do case conn.host do &quot;en.example.com&quot; -&gt; &quot;en&quot; &quot;nl.example.com&quot; -&gt; &quot;nl&quot; _ -&gt; case conn.path_info do [&quot;en&quot; | _] -&gt; &quot;en&quot; [&quot;nl&quot; | _] -&gt; &quot;nl&quot; _ -&gt; &quot;en&quot; end end end options = PutLocale.init(find_locale: &amp;find_locale/1) conn = conn(:get, &quot;/nl/hallo&quot;) conn = PutLocale.call(conn, options) assert conn.assigns == %{locale: &quot;nl&quot;} assert Gettext.get_locale == &quot;nl&quot;"},{"ref":"readme.html#installation","title":"I18n Helpers - Installation","type":"extras","doc":"Add i18n_helpers for Elixir as a dependency in your mix.exs file: def deps do [ {:i18n_helpers, &quot;~&gt; 0.6.0&quot;} ] end"},{"ref":"readme.html#hexdocs","title":"I18n Helpers - HexDocs","type":"extras","doc":"HexDocs documentation can be found at https://hexdocs.pm/i18n_helpers."}]